"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AopPointcut = exports.AopFactory = void 0;
const instancefactory_1 = require("./instancefactory");
const aopproxy_1 = require("./aopproxy");
const errorfactory_1 = require("../tools/errorfactory");
const transactionmanager_1 = require("../database/transactionmanager");
const util_1 = require("../tools/util");
/**
 * aop 切点类
 */
class AopPointcut {
    /**
     * 构造器
     * @param id            切点id(唯一)
     * @param expressions   该切点对应的表达式数组，表达式为正则表达式串
     */
    constructor(id, expressions) {
        /**
         * 表达式数组（正则表达式）
         */
        this.expressions = [];
        /**
         * 通知数组
         */
        this.advices = [];
        this.id = id;
        if (expressions) {
            this.addExpression(expressions);
        }
    }
    /**
     * 匹配方法是否满足表达式
     * @param instanceName  实例名
     * @param methodName    待检测方法
     * @returns             匹配返回true，否则返回false
     */
    match(instanceName, methodName) {
        for (let exp of this.expressions) {
            if (exp.test(instanceName + '.' + methodName)) {
                return true;
            }
        }
        return false;
    }
    /**
     * 给切点添加通知
     * @param advice    通知对象
     */
    addAdvice(advice) {
        this.advices.push(advice);
    }
    /**
     * 添加表达式串
     * @param expr  表达式串或数组
     */
    addExpression(expr) {
        if (!Array.isArray(expr)) {
            expr = [expr];
        }
        expr.forEach((item) => {
            if (typeof item !== 'string') {
                throw new errorfactory_1.NoomiError("2001");
            }
            this.expressions.push(util_1.Util.toReg(item));
        });
    }
}
exports.AopPointcut = AopPointcut;
/**
 * Aop工厂
 * 用于管理所有切面、切点
 */
class AopFactory {
    /**
     * 注册切面
     * @param className     切面类名
     * @since 1.0.0
     */
    static registAspect(className) {
        if (!this.registAspectMap.has(className)) {
            return;
        }
        this.registAspectMap.get(className).isAspect = true;
    }
    /**
     * 注册切点
     * @param cfg   pointcut配置
     * @since 1.0.0
     */
    static registPointcut(cfg) {
        let pc = this.getRegistPointcut(cfg.className, cfg.id, true);
        if (cfg.expressions) {
            pc.expressions = pc.expressions.concat(cfg.expressions);
        }
    }
    /**
     * 注册advice
     * @param cfg   advice配置
     * @since 1.0.0
     */
    static registAdvice(cfg) {
        if (!this.registAspectMap.has(cfg.className)) {
            return;
        }
        let pc = this.registAspectMap.get(cfg.className)[cfg.pointcutId];
        if (!pc) {
            return;
        }
        delete cfg.className;
        pc.advices.push(cfg);
    }
    /**
     * 从registAspectMap中获取注册的pointcut配置
     * @param className     类名
     * @param pointcutId    切点id
     * @param create        如果不存在，是否创建，如果为true，则创建，默认false
     * @returns             pointcut配置项
     * @since 1.0.0
     */
    static getRegistPointcut(className, pointcutId, create) {
        let pc;
        if (this.registAspectMap.has(className)) {
            let obj = this.registAspectMap.get(className);
            pc = obj[pointcutId];
        }
        else if (create) {
            this.registAspectMap.set(className, {});
        }
        if (!pc && create) { //新建pointcut配置项
            let obj = this.registAspectMap.get(className);
            pc = {
                advices: [],
                expressions: []
            };
            obj[pointcutId] = pc;
        }
        return pc;
    }
    /**
     * 处理实例切面
     * 把注册的切面添加到切面列表
     * @param instanceName  实例名
     * @param className     类名
     * @since 1.0.0
     */
    static handleInstanceAspect(instanceName, className) {
        if (!this.registAspectMap.has(className)) {
            return;
        }
        let cfg = this.registAspectMap.get(className);
        if (!cfg.isAspect) { //非aspect，不处理
            return;
        }
        delete cfg.isAspect;
        Object.keys(cfg).forEach(item => {
            let o = cfg[item];
            //新建pointcut
            let pc = new AopPointcut(item, o.expressions);
            //加入切点集
            this.pointcuts.set(item, pc);
            //为切点添加advice
            o.advices.forEach(item1 => {
                //设置实例或实例名
                item1.instance = instanceName;
                //添加advice
                pc.addAdvice(item1);
            });
        });
        this.aspects.set(instanceName, cfg);
        //删除已处理的class
        this.registAspectMap.delete(className);
    }
    /**
     * 为切点添加表达式
     * @param className     切点类名
     * @param pointcutId    切点id
     * @param expression    表达式或数组
     */
    static addExpression(className, pointcutId, expression) {
        // let pc = this.getRegistPointcut(className,pointcutId,true);
        let pc = this.pointcuts.get(pointcutId);
        if (!pc) {
            return;
        }
        pc.addExpression(expression);
    }
    /**
     * 更新aop匹配的方法代理，为所有aop匹配的方法设置代理
     */
    static updMethodProxy() {
        if (!this.pointcuts || this.pointcuts.size === 0) {
            return;
        }
        //遍历pointcut
        let pc;
        for (pc of this.pointcuts.values()) {
            let reg;
            //遍历expression
            for (reg of pc.expressions) {
                this.addProxyByExpression(reg);
            }
        }
    }
    /**
     * 通过正则式给方法加代理
     * @param expr          表达式正则式
     */
    static addProxyByExpression(expr) {
        //遍历instance factory设置aop代理
        let insFac = instancefactory_1.InstanceFactory.getFactory();
        for (let insName of insFac.keys()) {
            //先检测instanceName
            let instance = instancefactory_1.InstanceFactory.getInstance(insName);
            if (instance) {
                Object.getOwnPropertyNames(instance.__proto__).forEach(key => {
                    //给方法设置代理，constructor 不需要代理
                    if (key === 'constructor' || typeof (instance[key]) !== 'function') {
                        return;
                    }
                    let method = insName + '.' + key;
                    //已代理过，不再代理
                    if (this.proxyMethodMap.has(method)) {
                        return;
                    }
                    //实例名+方法符合aop正则表达式
                    if (expr.test(method)) {
                        instance[key] = aopproxy_1.AopProxy.invoke(insName, key, instance[key], instance);
                        this.proxyMethodMap.set(method, true);
                    }
                });
            }
        }
    }
    /**
     * 获取切点
     * @param instanceName  实例名
     * @param methodName    方法名
     * @returns             切点数组
     */
    static getPointcut(instanceName, methodName) {
        // 遍历iterator
        let a = [];
        for (let p of this.pointcuts.values()) {
            if (p.match(instanceName, methodName)) {
                a.push(p);
            }
        }
        return a;
    }
    /**
     * 根据id获取切点
     * @param pointcutId    切点id
     * @returns             切点对象
     */
    static getPointcutById(pointcutId) {
        return this.pointcuts.get(pointcutId);
    }
    /**
     * 获取advices
     * @param instanceName  实例名
     * @param methodName    方法名
     * @return              {
     *                          before:[{instance:切面实例,method:切面方法},...]
     *                          after:[{instance:切面实例,method:切面方法},...]
     *                          return:[{instance:切面实例,method:切面方法},...]
     *                          throw:[{instance:切面实例,method:切面方法},...]
     *                      }
     */
    static getAdvices(instanceName, methodName) {
        let pointcuts = this.getPointcut(instanceName, methodName);
        if (pointcuts.length === 0) {
            return null;
        }
        let beforeArr = [];
        let afterArr = [];
        let throwArr = [];
        let returnArr = [];
        let pointcut;
        let hasTransaction = false;
        for (pointcut of pointcuts) {
            if (pointcut.id === transactionmanager_1.TransactionManager.pointcutId) {
                hasTransaction = true;
                continue;
            }
            pointcut.advices.forEach(aop => {
                let ins = typeof aop.instance === 'string' ?
                    instancefactory_1.InstanceFactory.getInstance(aop.instance) : aop.instance;
                switch (aop.type) {
                    case 'before':
                        beforeArr.push({
                            instance: ins,
                            method: aop.method
                        });
                        return;
                    case 'after':
                        afterArr.push({
                            instance: ins,
                            method: aop.method
                        });
                        return;
                    case 'around':
                        beforeArr.push({
                            instance: ins,
                            method: aop.method
                        });
                        afterArr.push({
                            instance: ins,
                            method: aop.method
                        });
                        return;
                    case 'after-return':
                        returnArr.push({
                            instance: ins,
                            method: aop.method
                        });
                        return;
                    case 'after-throw':
                        throwArr.push({
                            instance: ins,
                            method: aop.method
                        });
                }
            });
        }
        return {
            hasTransaction: hasTransaction,
            before: beforeArr,
            after: afterArr,
            throw: throwArr,
            return: returnArr
        };
    }
}
exports.AopFactory = AopFactory;
/**
 * 切面map，用于存储所有切面
 */
AopFactory.aspects = new Map();
/**
 * 切点map，用于存储所有切点
 */
AopFactory.pointcuts = new Map();
/**
 * 已代理方法map，键为instanctName.methodName，避免重复代理
 * @since 0.4.4
 */
AopFactory.proxyMethodMap = new Map();
/**
 * 注册切面map，键为className,值为
 *          {
 *              isAspect:true,  //避免用了pointcut，但是未使用Aspect注解
 *              pointCutId1:{expressions:Array<string>,advices:{type:类型,method:方法名}},
 *              ...
 *          }
 */
AopFactory.registAspectMap = new Map();
//# sourceMappingURL=aopfactory.js.map