/**
 * Aop通知类型
 */
interface IAopAdvice {
    /**
     * 切点
     */
    pointcutId?: string;
    /**
     * 实例名或实例对象
     */
    instance?: any;
    /**
     * 类名
     */
    className?: string;
    /**
     * 通知类型 (before,after,after-return,after-throw,around)
     */
    type: string;
    /**
     * 对应的切面方法
     */
    method: string;
}
/**
 * Aop切面类型
 */
interface IAopAspect {
    /**
     * 实例名
     */
    instance?: string;
    /**
     * 切点数组
     */
    pointcuts: Array<AopPointcut>;
    /**
     * 通知数组
     */
    advices: Array<IAopAdvice>;
}
/**
 * 切点数据对象
 */
interface IPointcut {
    /**
     * 类名
     */
    className: string;
    /**
     * 切点id
     */
    id: string;
    /**
     * 表达式串
     */
    expressions?: Array<string>;
}
/**
 * @exclude
 * aop文件配置对象
 */
interface IAopCfg {
    files: Array<string>;
    pointcuts: Array<IPointcut>;
    aspects: Array<IAopAspect>;
}
/**
 * aop 切点类
 */
declare class AopPointcut {
    /**
     * 切点id
     */
    id: string;
    /**
     * 表达式数组（正则表达式）
     */
    expressions: Array<RegExp>;
    /**
     * 通知数组
     */
    advices: Array<IAopAdvice>;
    /**
     * 构造器
     * @param id            切点id(唯一)
     * @param expressions   该切点对应的表达式数组，表达式为正则表达式串
     */
    constructor(id: string, expressions: Array<string>);
    /**
     * 匹配方法是否满足表达式
     * @param instanceName  实例名
     * @param methodName    待检测方法
     * @returns             匹配返回true，否则返回false
     */
    match(instanceName: string, methodName: string): boolean;
    /**
     * 给切点添加通知
     * @param advice    通知对象
     */
    addAdvice(advice: IAopAdvice): void;
    /**
     * 添加表达式串
     * @param expr  表达式串或数组
     */
    addExpression(expr: string | string[]): void;
}
/**
 * Aop工厂
 * 用于管理所有切面、切点
 */
declare class AopFactory {
    /**
     * 切面map，用于存储所有切面
     */
    static aspects: Map<string, IAopAspect>;
    /**
     * 切点map，用于存储所有切点
     */
    static pointcuts: Map<string, AopPointcut>;
    /**
     * 已代理方法map，键为instanctName.methodName，避免重复代理
     * @since 0.4.4
     */
    static proxyMethodMap: Map<string, boolean>;
    /**
     * 注册切面map，键为className,值为
     *          {
     *              isAspect:true,  //避免用了pointcut，但是未使用Aspect注解
     *              pointCutId1:{expressions:Array<string>,advices:{type:类型,method:方法名}},
     *              ...
     *          }
     */
    private static registAspectMap;
    /**
     * 注册切面
     * @param className     切面类名
     * @since 1.0.0
     */
    static registAspect(className: string): void;
    /**
     * 注册切点
     * @param cfg   pointcut配置
     * @since 1.0.0
     */
    static registPointcut(cfg: IPointcut): void;
    /**
     * 注册advice
     * @param cfg   advice配置
     * @since 1.0.0
     */
    static registAdvice(cfg: IAopAdvice): void;
    /**
     * 从registAspectMap中获取注册的pointcut配置
     * @param className     类名
     * @param pointcutId    切点id
     * @param create        如果不存在，是否创建，如果为true，则创建，默认false
     * @returns             pointcut配置项
     * @since 1.0.0
     */
    private static getRegistPointcut;
    /**
     * 处理实例切面
     * 把注册的切面添加到切面列表
     * @param instanceName  实例名
     * @param className     类名
     * @since 1.0.0
     */
    static handleInstanceAspect(instanceName: string, className: string): void;
    /**
     * 为切点添加表达式
     * @param className     切点类名
     * @param pointcutId    切点id
     * @param expression    表达式或数组
     */
    static addExpression(className: string, pointcutId: string, expression: string | Array<string>): void;
    /**
     * 更新aop匹配的方法代理，为所有aop匹配的方法设置代理
     */
    static updMethodProxy(): void;
    /**
     * 通过正则式给方法加代理
     * @param expr          表达式正则式
     */
    private static addProxyByExpression;
    /**
     * 获取切点
     * @param instanceName  实例名
     * @param methodName    方法名
     * @returns             切点数组
     */
    static getPointcut(instanceName: string, methodName: string): Array<AopPointcut>;
    /**
     * 根据id获取切点
     * @param pointcutId    切点id
     * @returns             切点对象
     */
    static getPointcutById(pointcutId: string): AopPointcut;
    /**
     * 获取advices
     * @param instanceName  实例名
     * @param methodName    方法名
     * @return              {
     *                          before:[{instance:切面实例,method:切面方法},...]
     *                          after:[{instance:切面实例,method:切面方法},...]
     *                          return:[{instance:切面实例,method:切面方法},...]
     *                          throw:[{instance:切面实例,method:切面方法},...]
     *                      }
     */
    static getAdvices(instanceName: string, methodName: string): object;
}
export { AopFactory, IAopAdvice, IAopAspect, AopPointcut, IAopCfg, IPointcut };
